package com.gci.boot.modules.basedata.controller;


import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.constant.CommonConstant;
import com.gci.boot.modules.basedata.entity.TbExternalAddressBook;
import com.gci.boot.modules.basedata.service.ITbExternalAddressBookService;

import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 *  外部通讯录 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-08-24
 */
@Slf4j
@RestController
@RequestMapping("/basedata/externalAddressBook")
public class TbExternalAddressBookController {

	@Autowired
	private ITbExternalAddressBookService tbExternalAddressBookService;


	@AddSysLog(value = "外部通讯录-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbExternalAddressBook item) {
        LambdaQueryWrapper<TbExternalAddressBook> queryWrapper = QueryGenerator.initQueryWrapper(TbExternalAddressBook.class, item);
		queryWrapper.eq(TbExternalAddressBook::getDelFlag, CommonConstant.DEL_FLAG_0);
		/*LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		if(sysUser!=null) {
			queryWrapper.likeRight(StringUtils.hasLength(sysUser.getOrgCode()),TbExternalAddressBook::getSysOrgCode, sysUser.getOrgCode());
		}*/
		queryWrapper.orderByDesc(TbExternalAddressBook::getCreateTime);
		List<TbExternalAddressBook> list = tbExternalAddressBookService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "外部通讯录-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbExternalAddressBook item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        LambdaQueryWrapper<TbExternalAddressBook> queryWrapper = QueryGenerator.initQueryWrapper(TbExternalAddressBook.class, item);
		queryWrapper.eq(TbExternalAddressBook::getDelFlag, CommonConstant.DEL_FLAG_0);
		/*LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		if(sysUser!=null) {
			queryWrapper.likeRight(StringUtils.hasLength(sysUser.getOrgCode()),TbExternalAddressBook::getSysOrgCode, sysUser.getOrgCode());
		}*/
		queryWrapper.orderByDesc(TbExternalAddressBook::getCreateTime);
		Page<TbExternalAddressBook> page = new Page<TbExternalAddressBook>(pageNo, pageSize);
		IPage<TbExternalAddressBook> pageList = tbExternalAddressBookService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "外部通讯录-新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbExternalAddressBook item){
		tbExternalAddressBookService.save(item);
		return Result.ok();
	}

	@AddSysLog(value = "外部通讯录-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbExternalAddressBook item){
		tbExternalAddressBookService.updateById(item);
		return Result.ok();
	}

	@AddSysLog(value = "外部通讯录-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbExternalAddressBook item){
		tbExternalAddressBookService.removeById(item.getId());
		return Result.ok();
	}
	

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
	@AddSysLog(value = "外部通讯录-导出")
	@GetMapping("/simple/export")
	public Result<?> export(TbExternalAddressBook item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "外部通讯录.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbExternalAddressBook> list = (List<TbExternalAddressBook>)result.getResult();
			EasyExcel.write(output, TbExternalAddressBook.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

	@AddSysLog(value = "外部通讯录-分页导出")
	@GetMapping("/multi/export")
	public Result<?> multiExport(TbExternalAddressBook item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "外部通讯录.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbExternalAddressBook.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbExternalAddressBook> page = (IPage<TbExternalAddressBook>)result.getResult();
					List<TbExternalAddressBook> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}


	/**
	 * 多次读入
	 * @param request
	 * @return
	 */
	@AddSysLog(value = "外部通讯录-导入")
	@PostMapping("/simple/import")
	public Result<?> simpleImport(HttpServletRequest request) {
		MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
		Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
		for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
			MultipartFile file = entity.getValue();// 获取上传文件对象
			try(InputStream input = file.getInputStream()) {
				List<TbExternalAddressBook> list = EasyExcel.read(input).head(TbExternalAddressBook.class).sheet().doReadSync();
				log.info("数据量：{}",list.size());
				tbExternalAddressBookService.saveBatch(list);
			}catch (IOException e){
				log.error(e.getMessage(), e);
				return Result.error("导入错误！");
			}
		}
		return Result.ok();
	}
	
}
