package com.gci.boot.modules.basedata.controller;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.common.dict.excel.MyExcelDictHandleImpl;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.modules.basedata.entity.TbCustomerFollowTemplate;
import com.gci.boot.modules.basedata.entity.TbInspectionTemplate;
import com.gci.boot.modules.basedata.entity.TbInspectionTemplateProject;
import com.gci.boot.modules.basedata.service.ITbInspectionTemplateProjectService;
import com.gci.boot.modules.basedata.service.ITbInspectionTemplateService;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Arrays;

/**
 * <p>
 * 巡检模板表 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-09-09
 */
@RestController
@RequestMapping("/basedata/inspectionTemplate")
public class TbInspectionTemplateController {
    @Autowired
    private ITbInspectionTemplateService iTbInspectionTemplateService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog(value = "巡检模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(TbInspectionTemplate inspectionTemplate,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            HttpServletRequest req) {
        Page<TbInspectionTemplate> page = new Page<TbInspectionTemplate>(pageNo, pageSize);
        IPage<TbInspectionTemplate> pageList = iTbInspectionTemplateService.pageBuild(page, inspectionTemplate);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog(value = "巡检模板-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbInspectionTemplate tbInspectionTemplate) {
        iTbInspectionTemplateService.saveItem(tbInspectionTemplate);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @AddSysLog(value = "巡检模板-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbInspectionTemplate tbInspectionTemplate) {
        iTbInspectionTemplateService.updateItem(tbInspectionTemplate);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @AddSysLog(value = "巡检模板-删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iTbInspectionTemplateService.removeItemById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
    @AddSysLog(value = "巡检模板-批量删除")
//    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iTbInspectionTemplateService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     *  导出
     */
    @AddSysLog(value = "巡检模板-导出")
    @GetMapping("/export")
    public Result<?> export(HttpServletResponse response, TbInspectionTemplate inspectionTemplate){
        try (OutputStream output = response.getOutputStream()){
            String fileName = "巡检模板.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
            Page<TbInspectionTemplate> page = new Page<TbInspectionTemplate>(1, -1);
            IPage<TbInspectionTemplate> pageList = iTbInspectionTemplateService.pageBuild(page, inspectionTemplate);
            ExportParams exportParams = new ExportParams();
            exportParams.setDictHandler(new MyExcelDictHandleImpl());
            Workbook sheets = ExcelExportUtil.exportExcel(exportParams, TbInspectionTemplate.class, pageList.getRecords());
            sheets.write(output);
        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }
}
