package com.gci.boot.modules.customer.controller;


import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.gci.boot.annotation.permission.DataPermission;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.util.DateUtils;
import com.gci.boot.common.util.MinioUtils;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.enums.FileExtension;
import com.gci.boot.modules.customer.entity.TbCustomerOpinion;
import com.gci.boot.modules.customer.service.ITbCustomerOpinionHandleService;
import com.gci.boot.modules.customer.service.ITbCustomerOpinionService;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 客户反馈意见 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-10-16
 */
@Slf4j
@RestController
@RequestMapping("/customer/customerOpinion")
public class TbCustomerOpinionController {

	@Autowired
	private ITbCustomerOpinionService tbCustomerOpinionService;
	@Autowired
	private ITbCustomerOpinionHandleService tbCustomerOpinionHandleService;
	@Autowired
    private MinioUtils minioUtils;

	@AddSysLog(value = "客户反馈意见列表查询")
	@GetMapping("/list")
	@DataPermission(item="tb_customer_opinion.projectId", sqlIds = {"com.gci.boot.modules.customer.mapper.TbCustomerOpinionMapper.selectList"})
	public Result<?> list(TbCustomerOpinion item) {
		LambdaQueryWrapper<TbCustomerOpinion> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerOpinion.class, item);
		queryWrapper.ge(item.getRegisterTimeStart()!=null,TbCustomerOpinion::getRegisterTime, item.getRegisterTimeStart());
		queryWrapper.le(item.getRegisterTimeEnd()!=null,TbCustomerOpinion::getRegisterTime, DateUtils.getDayEnd(item.getRegisterTimeEnd()));
		queryWrapper.orderByDesc(TbCustomerOpinion::getCreateTime);
		List<TbCustomerOpinion> list = tbCustomerOpinionService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "客户反馈意见分页查询")
	@GetMapping("/page")
	@DataPermission(item="tb_customer_opinion.projectId", sqlIds = {"com.gci.boot.modules.customer.mapper.TbCustomerOpinionMapper.selectPage"})
	public Result<?> page(TbCustomerOpinion item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
		LambdaQueryWrapper<TbCustomerOpinion> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerOpinion.class, item);
		queryWrapper.ge(item.getRegisterTimeStart()!=null,TbCustomerOpinion::getRegisterTime, item.getRegisterTimeStart());
		queryWrapper.le(item.getRegisterTimeEnd()!=null,TbCustomerOpinion::getRegisterTime, DateUtils.getDayEnd(item.getRegisterTimeEnd()));
		queryWrapper.orderByDesc(TbCustomerOpinion::getCreateTime);
		Page<TbCustomerOpinion> page = new Page<TbCustomerOpinion>(pageNo, pageSize);
		IPage<TbCustomerOpinion> pageList = tbCustomerOpinionService.page(page,queryWrapper);
		for(TbCustomerOpinion template:pageList.getRecords()) {
			tbCustomerOpinionHandleService.getHandlesByOpinionId(template);
		}
		return Result.ok(pageList);
	}

	@AddSysLog(value = "客户反馈意见新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbCustomerOpinion item){
		tbCustomerOpinionService.save(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户反馈意见编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbCustomerOpinion item){
		tbCustomerOpinionService.updateById(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户反馈意见删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbCustomerOpinion item){
		tbCustomerOpinionService.removeById(item.getId());
		return Result.ok();
	}

    /**
     * 文件上传
     */
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileValid(file, FileExtension.getCommonFileTypes()));
    }

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
    @AddSysLog(value = "客户反馈意见导出")
	@GetMapping("/simple/export")
	@DataPermission(item="tb_customer_opinion.projectId", sqlIds = {"com.gci.boot.modules.customer.mapper.TbCustomerOpinionMapper.selectList"})
	public Result<?> export(TbCustomerOpinion item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "客户回访台账.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbCustomerOpinion> list = (List<TbCustomerOpinion>)result.getResult();
			EasyExcel.write(output, TbCustomerOpinion.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

    @AddSysLog(value = "客户反馈意见分页导出")
	@GetMapping("/multi/export")
	@DataPermission(item="tb_customer_opinion.projectId", sqlIds = {"com.gci.boot.modules.customer.mapper.TbCustomerOpinionMapper.selectPage"})
	public Result<?> multiExport(TbCustomerOpinion item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "客户回访台账.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbCustomerOpinion.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbCustomerOpinion> page = (IPage<TbCustomerOpinion>)result.getResult();
					List<TbCustomerOpinion> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

}
