package com.gci.boot.modules.customer.controller;


import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.modules.customer.entity.TbCustomerOpinionHandle;
import com.gci.boot.modules.customer.service.ITbCustomerOpinionHandleService;
import com.gci.boot.modules.customer.service.ITbCustomerOpinionService;

import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 客户反馈意见处理 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-10-16
 */
@Slf4j
@RestController
@RequestMapping("/customer/customerOpinionHandle")
public class TbCustomerOpinionHandleController {

	@Autowired
	private ITbCustomerOpinionHandleService tbCustomerFollowPartService;

	@Autowired
	private ITbCustomerOpinionService tbCustomerOpinionService;
	
	@AddSysLog(value = "客户反馈意见处理-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbCustomerOpinionHandle item) {
		LambdaQueryWrapper<TbCustomerOpinionHandle> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerOpinionHandle.class, item);
		queryWrapper.orderByAsc(TbCustomerOpinionHandle::getCreateTime);
		List<TbCustomerOpinionHandle> list = tbCustomerFollowPartService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "客户反馈意见处理-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbCustomerOpinionHandle item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<TbCustomerOpinionHandle> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerOpinionHandle.class, item);
		queryWrapper.orderByAsc(TbCustomerOpinionHandle::getCreateTime);
		Page<TbCustomerOpinionHandle> page = new Page<TbCustomerOpinionHandle>(pageNo, pageSize);
		IPage<TbCustomerOpinionHandle> pageList = tbCustomerFollowPartService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "客户反馈意见处理-新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbCustomerOpinionHandle item){
		tbCustomerFollowPartService.save(item);
		//满意更新反馈意见状态
		tbCustomerOpinionService.updateStatus(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户反馈意见处理-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbCustomerOpinionHandle item){
		tbCustomerFollowPartService.updateById(item);
		//满意更新反馈意见状态
		tbCustomerOpinionService.updateStatus(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户反馈意见处理-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbCustomerOpinionHandle item){
		tbCustomerFollowPartService.removeById(item.getId());
		return Result.ok();
	}

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
    @AddSysLog(value = "客户反馈意见处理-导出")
	@GetMapping("/simple/export")
	public Result<?> export(TbCustomerOpinionHandle item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "客户反馈意见处理.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbCustomerOpinionHandle> list = (List<TbCustomerOpinionHandle>)result.getResult();
			EasyExcel.write(output, TbCustomerOpinionHandle.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

    @AddSysLog(value = "客户反馈意见处理-分页导出")
	@GetMapping("/multi/export")
	public Result<?> multiExport(TbCustomerOpinionHandle item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "客户反馈意见处理.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbCustomerOpinionHandle.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbCustomerOpinionHandle> page = (IPage<TbCustomerOpinionHandle>)result.getResult();
					List<TbCustomerOpinionHandle> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

}
