package com.gci.boot.modules.driver.controller;


import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.util.StrUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.annotation.permission.DataPermission;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.util.DateUtils;
import com.gci.boot.common.util.MinioUtils;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.enums.FileExtension;
import com.gci.boot.modules.driver.entity.TbDriverSignIn;
import com.gci.boot.modules.driver.service.ITbDriverSignInService;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 司机打卡 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-13
 */
@Slf4j
@RestController
@RequestMapping("/driver/driverSignIn")
public class TbDriverSignInController {

	@Autowired
	private ITbDriverSignInService tbDriverSignInService;
	@Autowired
    private MinioUtils minioUtils;

	@AddSysLog(value = "司机打卡-列表查询")
	@GetMapping("/list")
	@DataPermission(item="tb_driver_sign_in.projectId", sqlIds = {"com.gci.boot.modules.driver.mapper.TbDriverSignInMapper.selectList"})
	public Result<?> list(TbDriverSignIn item) {
		LambdaQueryWrapper<TbDriverSignIn> queryWrapper = QueryGenerator.initQueryWrapper(TbDriverSignIn.class, item);
		queryWrapper.ge(item.getSignInTimeStart()!=null,TbDriverSignIn::getSignInTime, item.getSignInTimeStart());
		queryWrapper.le(item.getSignInTimeEnd()!=null,TbDriverSignIn::getSignInTime, item.getSignInTimeEnd());
		queryWrapper.le(StrUtil.isNotBlank(item.getPlateNumber()),TbDriverSignIn::getPlateNumber,item.getPlateNumber());
		queryWrapper.orderByDesc(TbDriverSignIn::getSignInTime);
		List<TbDriverSignIn> list = tbDriverSignInService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "司机打卡-分页查询")
	@GetMapping("/page")
	@DataPermission(item="tb_driver_sign_in.projectId", sqlIds = {"com.gci.boot.modules.driver.mapper.TbDriverSignInMapper.selectPage"})
	public Result<?> page(TbDriverSignIn item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<TbDriverSignIn> queryWrapper = QueryGenerator.initQueryWrapper(TbDriverSignIn.class, item);
		queryWrapper.ge(item.getSignInTimeStart()!=null,TbDriverSignIn::getSignInTime, item.getSignInTimeStart());
		queryWrapper.le(item.getSignInTimeEnd()!=null,TbDriverSignIn::getSignInTime, item.getSignInTimeEnd());
		queryWrapper.le(StrUtil.isNotBlank(item.getPlateNumber()),TbDriverSignIn::getPlateNumber,item.getPlateNumber());
		queryWrapper.orderByDesc(TbDriverSignIn::getSignInTime);
		Page<TbDriverSignIn> page = new Page<TbDriverSignIn>(pageNo, pageSize);
		IPage<TbDriverSignIn> pageList = tbDriverSignInService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "司机打卡-新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbDriverSignIn item){
		item.setOpType("3");
		item.setSignInDate(DateUtils.getDayStart(item.getSignInTime()));
		tbDriverSignInService.save(item);
		return Result.ok();
	}

	@AddSysLog(value = "司机打卡-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbDriverSignIn item){
		item.setSignInDate(DateUtils.getDayStart(item.getSignInTime()));
		tbDriverSignInService.updateById(item);
		return Result.ok();
	}

	@AddSysLog(value = "司机打卡-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbDriverSignIn item){
		tbDriverSignInService.removeById(item.getId());
		return Result.ok();
	}

    /**
     * 文件上传
     */
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileValid(file, FileExtension.getCommonFileTypes()));
    }

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
    @AddSysLog(value = "司机打卡-导出")
	@GetMapping("/simple/export")
	@DataPermission(item="tb_driver_sign_in.projectId", sqlIds = {"com.gci.boot.modules.driver.mapper.TbDriverSignInMapper.selectList"})
	public Result<?> export(TbDriverSignIn item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "司机打卡.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbDriverSignIn> list = (List<TbDriverSignIn>)result.getResult();
			EasyExcel.write(output, TbDriverSignIn.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

    @AddSysLog(value = "司机打卡-分页导出")
	@GetMapping("/multi/export")
	@DataPermission(item="tb_driver_sign_in.projectId", sqlIds = {"com.gci.boot.modules.driver.mapper.TbDriverSignInMapper.selectPage"})
	public Result<?> multiExport(TbDriverSignIn item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "司机打卡.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbDriverSignIn.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbDriverSignIn> page = (IPage<TbDriverSignIn>)result.getResult();
					List<TbDriverSignIn> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

	/**
	 * 多次读入
	 * @param request
	 * @return
	 */
	@AddSysLog(value = "司机打卡-导入")
	@PostMapping("/simple/import")
	public Result<?> simpleImport(HttpServletRequest request) {
		MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
		Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
		for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
			MultipartFile file = entity.getValue();// 获取上传文件对象
			try(InputStream input = file.getInputStream()) {
				List<TbDriverSignIn> list = EasyExcel.read(input).head(TbDriverSignIn.class).sheet().doReadSync();
				tbDriverSignInService.saveItemBatch(list);
			}catch (IOException e){
				log.error(e.getMessage(), e);
				return Result.error("导入错误！");
			}
		}
		return Result.ok();
	}

}
