package com.gci.boot.modules.gis.controller;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.annotation.permission.DataPermission;
import com.gci.boot.common.dict.excel.MyExcelDictHandleImpl;
import com.gci.boot.common.vo.Result;
import com.gci.boot.modules.gis.entity.TbGisChargeLog;
import com.gci.boot.modules.gis.query.GisChargeLogQuery;
import com.gci.boot.modules.gis.service.ITbGisChargeLogService;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Arrays;

/**
 * <p>
 * 充电记录表 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-09-01
 */
@RestController
@RequestMapping("/gis/chargeLog")
public class TbGisChargeLogController {
    @Autowired
    private ITbGisChargeLogService iTbGisChargeLogService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @DataPermission(item = "tgcl.projectId", sqlIds = {"com.gci.boot.modules.gis.mapper.TbGisChargeLogMapper.pageBuild"})
    @GetMapping(value = "/list")
    public Result<?> queryPageList(GisChargeLogQuery query,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<TbGisChargeLog> page = new Page<TbGisChargeLog>(pageNo, pageSize);
        LambdaQueryWrapper<TbGisChargeLog> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbGisChargeLog.class, query);
        IPage<TbGisChargeLog> pageList = iTbGisChargeLogService.pageBuild(page, lambdaQueryWrapper);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody TbGisChargeLog tbGisChargeLog) {
        iTbGisChargeLogService.save(tbGisChargeLog);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody TbGisChargeLog tbGisChargeLog) {
        iTbGisChargeLogService.updateById(tbGisChargeLog);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iTbGisChargeLogService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
//    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iTbGisChargeLogService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 导出
     */
    @GetMapping("/export")
    @DataPermission(item = "tgcl.projectId", sqlIds = {"com.gci.boot.modules.gis.mapper.TbGisChargeLogMapper.pageBuild"})
    public Result<?> export(HttpServletResponse response, GisChargeLogQuery query) {
        try (OutputStream output = response.getOutputStream()) {
            String fileName = "充电记录.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
            Page<TbGisChargeLog> page = new Page<TbGisChargeLog>(1, -1);
            LambdaQueryWrapper<TbGisChargeLog> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbGisChargeLog.class, query);
            IPage<TbGisChargeLog> pageList = iTbGisChargeLogService.pageBuild(page, lambdaQueryWrapper);
            ExportParams exportParams = new ExportParams();
            exportParams.setDictHandler(new MyExcelDictHandleImpl());
            Workbook sheets = ExcelExportUtil.exportExcel(exportParams, TbGisChargeLog.class, pageList.getRecords());
            sheets.write(output);
        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }
}
