package com.gci.boot.modules.gis.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.gci.boot.common.dict.annotation.Dict;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * <p>
 * 充电记录表
 * </p>
 *
 * @author 2464
 * @since 2023-09-01
 */
@Getter
@Setter
@TableName("tb_gis_charge_log")
@ApiModel(value = "TbGisChargeLog对象", description = "充电记录表")
public class TbGisChargeLog implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty("车牌号")
    @Excel(name = "车牌号")
    private String numberPlate;


    @ApiModelProperty("车辆品牌")
    @Dict(dicCode = "vehicle_brand")
    @Excel(name = "车辆品牌",dict = "vehicle_brand")
    @TableField(exist = false)
    private String vehicleBrand;

    @ApiModelProperty("车辆型号")
    @Excel(name = "车辆型号")
    @TableField(exist = false)
    private String vehicleModel;

    @ApiModelProperty("车辆种类")
    @Dict(dicCode = "vehicle_type")
    @Excel(name = "车辆种类",dict = "vehicle_type")
    @TableField(exist = false)
    private String vehicleType;

    @ApiModelProperty("所属车商")
    @Dict(dicCode = "id",dictTable = "tb_vehicle_supplier_archives",dicText = "shortened")
    @Excel(name = "所属车商",dict = "tb_vehicle_supplier_archives,shortened,id")
    @TableField(exist = false)
    private String dealer;

    @ApiModelProperty("所属项目id")
    private String projectId;

    @ApiModelProperty("所属项目名称")
    @Excel(name = "所属项目")
    private String projectName;

    @ApiModelProperty("VIN 码")
    @Excel(name = "VIN码")
    private String vinCode;

    @ApiModelProperty("充电开始时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Excel(name = "充电开始时间",format = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;

    @ApiModelProperty("充电结束时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Excel(name = "充电结束时间",format = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;

    @ApiModelProperty("开始充电时的 SOC %")
    @Excel(name = "开始充电时的 SOC %")
    private BigDecimal startSoc;

    @ApiModelProperty("结束充电时的 SOC %")
    @Excel(name = "结束充电时的 SOC %")
    private BigDecimal endSoc;

    @ApiModelProperty("充电比 %")
    @Excel(name = "充电比 %")
    private BigDecimal chargingEfficiency;

    @ApiModelProperty("充电量 kwh")
    @Excel(name = "充电量 kwh")
    private BigDecimal chargingEnergy;

    @ApiModelProperty("电池总电量")
    @Excel(name = "电池总电量")
    private BigDecimal totalBatteryEnergy;

    @ApiModelProperty("经度")
    @Excel(name = "经度")
    private BigDecimal lon;

    @ApiModelProperty("纬度")
    @Excel(name = "纬度")
    private BigDecimal lat;

    @ApiModelProperty("高德经度")
    @Excel(name = "高德经度")
    private BigDecimal gdLon;

    @ApiModelProperty("高德纬度")
    @Excel(name = "高德纬度")
    private BigDecimal gdLat;

    @ApiModelProperty("采集时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Excel(name = "采集时间")
    private Date collectionTime;

    @ApiModelProperty("入库时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @ApiModelProperty("持续时间")
    @TableField(exist = false)
    @Excel(name = "持续时间")
    private String duration;

    @ApiModelProperty("充电预估费用")
    @Excel(name = "充电预估费用")
    private String estimatedCost;
}
