package com.gci.boot.modules.statis.controller;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.permission.DataPermission;
import com.gci.boot.common.dict.excel.MyExcelDictHandleImpl;
import com.gci.boot.common.vo.Result;
import com.gci.boot.modules.statis.service.VehicleRentalService;
import com.gci.boot.modules.statis.vo.VehicleRentalVo;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * @author 24642
 */
@Api(tags="车辆组件收入")
@Slf4j
@RestController
@RequestMapping("/statis/vehicleRental")
public class VehicleRentalController {
    @Autowired
    private VehicleRentalService vehicleRentalService;

    @GetMapping(value = "/list")
    @DataPermission(item="tb_contract.projectId", sqlIds = {"com.gci.boot.modules.projects.mapper.TbContractMapper.selectList"})
    public Result<?> queryPageList(VehicleRentalVo query,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<VehicleRentalVo> page = new Page<VehicleRentalVo>(pageNo, pageSize);
        IPage<VehicleRentalVo> pageList = vehicleRentalService.pageBuild(page,query);
        return Result.ok(pageList);
    }

    /**
     * 导出
     * @return
     */
    @GetMapping("/export")
    @DataPermission(item="tb_contract.projectId", sqlIds = {"com.gci.boot.modules.projects.mapper.TbContractMapper.selectList"})
    public Result<?> export(VehicleRentalVo query, HttpServletResponse response){
        try (OutputStream output = response.getOutputStream()){

            String fileName = "车辆租金收入.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
            Page<VehicleRentalVo> page = new Page<VehicleRentalVo>(1, -1);
            IPage<VehicleRentalVo> pageList = vehicleRentalService.pageBuild(page,query);
            ExportParams exportParams = new ExportParams();
            exportParams.setDictHandler(new MyExcelDictHandleImpl());
            Workbook sheets = ExcelExportUtil.exportExcel(exportParams, VehicleRentalVo.class, pageList.getRecords());
            sheets.write(output);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return Result.error("导出错误！");
        }
        return Result.ok();
    }
}
