package com.gci.boot.modules.vehicles.controller;


import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.gci.boot.annotation.permission.DataPermission;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.boot.annotation.Query.handle.QueryGenerator;
import com.gci.boot.common.log.annotation.AddSysLog;
import com.gci.boot.common.util.MinioUtils;
import com.gci.boot.common.validated.SaveValidated;
import com.gci.boot.common.validated.UpdateValidated;
import com.gci.boot.common.vo.Result;
import com.gci.boot.enums.FileExtension;
import com.gci.boot.modules.vehicles.entity.TbInsurance;
import com.gci.boot.modules.vehicles.entity.TbInsuranceRecord;
import com.gci.boot.modules.vehicles.service.ITbInsuranceRecordService;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 出险管理 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-22
 */
@Slf4j
@RestController
@RequestMapping("/vehicles/insuranceRecord")
public class TbInsuranceRecordController {

	@Autowired
	private ITbInsuranceRecordService tbInsuranceRecordService;
	@Autowired
    private MinioUtils minioUtils;

	@AddSysLog(value = "出险管理-列表查询")
	@GetMapping("/list")
	@DataPermission(item="tb_insurance_record.projectId", sqlIds = {"com.gci.boot.modules.vehicles.mapper.TbInsuranceRecordMapper.selectList"})
	public Result<?> list(TbInsuranceRecord item) {
		LambdaQueryWrapper<TbInsuranceRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbInsuranceRecord.class, item);
		queryWrapper.ge(item.getInsuranceDateStart()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateStart());
		queryWrapper.le(item.getInsuranceDateEnd()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateEnd());
		queryWrapper.orderByDesc(TbInsuranceRecord::getCreateTime);
		List<TbInsuranceRecord> list = tbInsuranceRecordService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "出险管理-分页查询")
	@GetMapping("/page")
	@DataPermission(item="tb_insurance_record.projectId", sqlIds = {"com.gci.boot.modules.vehicles.mapper.TbInsuranceRecordMapper.selectPage"})
	public Result<?> page(TbInsuranceRecord item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<TbInsuranceRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbInsuranceRecord.class, item);
		queryWrapper.ge(item.getInsuranceDateStart()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateStart());
		queryWrapper.le(item.getInsuranceDateEnd()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateEnd());
		queryWrapper.orderByDesc(TbInsuranceRecord::getCreateTime);
		Page<TbInsuranceRecord> page = new Page<TbInsuranceRecord>(pageNo, pageSize);
		IPage<TbInsuranceRecord> pageList = tbInsuranceRecordService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "出险管理-新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbInsuranceRecord item){
		tbInsuranceRecordService.saveItem(item);
		return Result.ok();
	}

	@AddSysLog(value = "出险管理-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbInsuranceRecord item){
		tbInsuranceRecordService.updateItem(item);
		return Result.ok();
	}

	@AddSysLog(value = "出险管理-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbInsuranceRecord item){
		tbInsuranceRecordService.removeById(item.getId());
		return Result.ok();
	}

    /**
     * 文件上传
     */
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileValid(file, FileExtension.getCommonFileTypes()));
    }

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
    @AddSysLog(value = "出险管理-导出")
	@GetMapping("/simple/export")
	@DataPermission(item="tb_insurance_record.projectId", sqlIds = {"com.gci.boot.modules.vehicles.mapper.TbInsuranceRecordMapper.selectList"})
	public Result<?> export(TbInsuranceRecord item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "司机打卡.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbInsuranceRecord> list = (List<TbInsuranceRecord>)result.getResult();
			EasyExcel.write(output, TbInsuranceRecord.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

    @AddSysLog(value = "出险管理-分页导出")
	@GetMapping("/multi/export")
	@DataPermission(item="tb_insurance_record.projectId", sqlIds = {"com.gci.boot.modules.vehicles.mapper.TbInsuranceRecordMapper.selectPage"})
	public Result<?> multiExport(TbInsuranceRecord item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "司机打卡.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbInsuranceRecord.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbInsuranceRecord> page = (IPage<TbInsuranceRecord>)result.getResult();
					List<TbInsuranceRecord> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

}
