
CREATE TABLE public.tb_gis_gps_analysis (
	id varchar(32) NOT NULL,
	project_id varchar(32) NOT NULL, -- 所属项目id
	number_plate varchar(32) NOT NULL, -- 车牌号
	vehicle_brand varchar(2) NULL, -- 车辆品牌
	vehicle_model varchar(32) NULL, -- 车辆型号
	vehicle_type varchar(2) NULL, -- 车辆种类
	own_date date NOT NULL, -- 所属日期 例如：2023-06-05
	record_type int2 NOT NULL, -- 记录类型 1-行驶未作业 2-行驶作业 3-停靠作业 4-停靠未作业
	start_time timestamp NULL, -- 开始时间
	end_time timestamp NULL, -- 结束时间
	duration int8 NOT NULL DEFAULT 0, -- 时长
	start_mile numeric(12, 2) NULL DEFAULT 0, -- 开始里程 km
	end_mile numeric(12, 2) NULL DEFAULT 0, -- 结束里程 km
	mile numeric(12, 2) NULL DEFAULT 0, -- 里程 km
	start_soc int2 NULL DEFAULT 0, -- 开始soc %
	end_soc int2 NULL DEFAULT 0, -- 结束soc %
	power_consume numeric(12, 2) NULL DEFAULT 0, -- 耗电量 kwh
	create_time timestamp NOT NULL, -- 入库时间
	CONSTRAINT pk_tb_gis_gps_analysis PRIMARY KEY (id)
);
CREATE INDEX tb_gis_gps_analysis_number_plate_idx ON public.tb_gis_gps_analysis USING btree (number_plate, own_date);
COMMENT ON TABLE public.tb_gis_gps_analysis IS '车辆gps记录分析表';

-- Column comments

COMMENT ON COLUMN public.tb_gis_gps_analysis.project_id IS '所属项目id';
COMMENT ON COLUMN public.tb_gis_gps_analysis.number_plate IS '车牌号';
COMMENT ON COLUMN public.tb_gis_gps_analysis.vehicle_brand IS '车辆品牌';
COMMENT ON COLUMN public.tb_gis_gps_analysis.vehicle_model IS '车辆型号';
COMMENT ON COLUMN public.tb_gis_gps_analysis.vehicle_type IS '车辆种类';
COMMENT ON COLUMN public.tb_gis_gps_analysis.own_date IS '所属日期 例如：2023-06-05';
COMMENT ON COLUMN public.tb_gis_gps_analysis.record_type IS '记录类型 1-行驶未作业 2-行驶作业 3-停靠作业 4-停靠未作业';
COMMENT ON COLUMN public.tb_gis_gps_analysis.start_time IS '开始时间';
COMMENT ON COLUMN public.tb_gis_gps_analysis.end_time IS '结束时间';
COMMENT ON COLUMN public.tb_gis_gps_analysis.duration IS '时长';
COMMENT ON COLUMN public.tb_gis_gps_analysis.start_mile IS '开始里程 km';
COMMENT ON COLUMN public.tb_gis_gps_analysis.end_mile IS '结束里程 km';
COMMENT ON COLUMN public.tb_gis_gps_analysis.mile IS '里程 km';
COMMENT ON COLUMN public.tb_gis_gps_analysis.start_soc IS '开始soc %';
COMMENT ON COLUMN public.tb_gis_gps_analysis.end_soc IS '结束soc %';
COMMENT ON COLUMN public.tb_gis_gps_analysis.power_consume IS '耗电量 kwh';
COMMENT ON COLUMN public.tb_gis_gps_analysis.create_time IS '入库时间';


ALTER TABLE public.tb_gis_gps_analysis ADD battery_capacity numeric(12, 2) NULL DEFAULT 0;
COMMENT ON COLUMN public.tb_gis_gps_analysis.battery_capacity IS '电池容量kwh';




-- 增加gps表状态
ALTER TABLE public.tb_gis_gps ADD is_work int2 NULL DEFAULT 0;

-- 增加gps表状态
ALTER TABLE public.tb_gis_runningbus ADD is_work int2 NULL DEFAULT -1;
COMMENT ON COLUMN public.tb_gis_runningbus.is_work IS '是否作业 0-否 1-是 -1无效';

-- 客户培训
-- public.tb_customer_training definition

-- Drop table

-- DROP TABLE public.tb_customer_training;

CREATE TABLE public.tb_customer_training (
                                             id varchar(32) NOT NULL, -- 主键id
                                             sys_org_code varchar(16) NULL, -- 所属机构
                                             create_time timestamp NULL, -- 创建时间
                                             update_time timestamp NULL, -- 更新时间
                                             create_by varchar(32) NULL, -- 创建人
                                             update_by varchar(32) NULL, -- 更新人
                                             project_id varchar(32) NULL, -- 所属项目
                                             contract_no varchar(64) NULL, -- 合同编号
                                             expected_start_time timestamp NULL, -- 预计开始时间
                                             training_type varchar(64) NULL, -- 培训类型
                                             trainees varchar(64) NULL, -- 培训对象
                                             actual_start_time timestamp NULL, -- 实际开始时间
                                             actual_end_time timestamp NULL, -- 培训结束时间
                                             lecturer varchar(64) NULL, -- 授课人
                                             teaching_content varchar(1024) NULL, -- 授课内容
                                             attachment text NULL, -- 附件
                                             CONSTRAINT tb_customer_training_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_customer_training IS '客户培训';

-- Column comments

COMMENT ON COLUMN public.tb_customer_training.id IS '主键id';
COMMENT ON COLUMN public.tb_customer_training.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_customer_training.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_customer_training.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_customer_training.create_by IS '创建人';
COMMENT ON COLUMN public.tb_customer_training.update_by IS '更新人';
COMMENT ON COLUMN public.tb_customer_training.project_id IS '所属项目';
COMMENT ON COLUMN public.tb_customer_training.contract_no IS '合同编号';
COMMENT ON COLUMN public.tb_customer_training.expected_start_time IS '预计开始时间';
COMMENT ON COLUMN public.tb_customer_training.training_type IS '培训类型';
COMMENT ON COLUMN public.tb_customer_training.trainees IS '培训对象';
COMMENT ON COLUMN public.tb_customer_training.actual_start_time IS '实际开始时间';
COMMENT ON COLUMN public.tb_customer_training.actual_end_time IS '培训结束时间';
COMMENT ON COLUMN public.tb_customer_training.lecturer IS '授课人';
COMMENT ON COLUMN public.tb_customer_training.teaching_content IS '授课内容';
COMMENT ON COLUMN public.tb_customer_training.attachment IS '附件';


-- public.tb_inspection_task definition

-- Drop table

-- DROP TABLE public.tb_inspection_task;

CREATE TABLE public.tb_inspection_task (
                                           id varchar(32) NOT NULL, -- 主键id
                                           sys_org_code varchar(16) NULL, -- 所属机构
                                           create_time timestamp NULL, -- 创建时间
                                           update_time timestamp NULL, -- 更新时间
                                           create_by varchar(32) NULL, -- 创建人
                                           update_by varchar(32) NULL, -- 更新人
                                           inspection_time timestamp NULL, -- 巡检时间
                                           inspection_type varchar(2) NULL, -- 巡检类型 1-线上巡检 2-回访巡检
                                           inspection_template_id varchar(32) NULL, -- 巡检项目Id
                                           handle_plan varchar(512) NULL, -- 处理方案
                                           handle_type varchar(2) NULL, -- 处理结果
                                           handle_time timestamp NULL, -- 处理时间
                                           source_type varchar(2) NULL DEFAULT 1, -- 来源（1用户，2计划）
                                           inspection_result varchar(2) NULL, -- 巡检结果
                                           remark varchar(512) NULL, -- 备注
                                           attachment text NULL, -- 附件
                                           follow_id varchar(32) NULL, -- 回访台账id
                                           inspection_user_id varchar(32) NULL, -- 处理人
                                           CONSTRAINT tb_inspection_records_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_inspection_task IS '巡检任务';

-- Column comments

COMMENT ON COLUMN public.tb_inspection_task.id IS '主键id';
COMMENT ON COLUMN public.tb_inspection_task.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_inspection_task.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_inspection_task.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_inspection_task.create_by IS '创建人';
COMMENT ON COLUMN public.tb_inspection_task.update_by IS '更新人';
COMMENT ON COLUMN public.tb_inspection_task.inspection_time IS '巡检时间';
COMMENT ON COLUMN public.tb_inspection_task.inspection_type IS '巡检类型 1-线上巡检 2-回访巡检';
COMMENT ON COLUMN public.tb_inspection_task.inspection_template_id IS '巡检项目Id';
COMMENT ON COLUMN public.tb_inspection_task.handle_plan IS '处理方案';
COMMENT ON COLUMN public.tb_inspection_task.handle_type IS '处理结果';
COMMENT ON COLUMN public.tb_inspection_task.handle_time IS '处理时间';
COMMENT ON COLUMN public.tb_inspection_task.source_type IS '来源（1用户，2计划）';
COMMENT ON COLUMN public.tb_inspection_task.inspection_result IS '巡检结果';
COMMENT ON COLUMN public.tb_inspection_task.remark IS '备注';
COMMENT ON COLUMN public.tb_inspection_task.attachment IS '附件';
COMMENT ON COLUMN public.tb_inspection_task.follow_id IS '回访台账id';
COMMENT ON COLUMN public.tb_inspection_task.inspection_user_id IS '处理人';


-- public.tb_inspection_task_part definition

-- Drop table

-- DROP TABLE public.tb_inspection_task_part;

CREATE TABLE public.tb_inspection_task_part (
                                                id varchar(32) NOT NULL, -- 主键id
                                                sys_org_code varchar(16) NULL, -- 所属机构
                                                create_time timestamp NULL, -- 创建时间
                                                update_time timestamp NULL, -- 更新时间
                                                create_by varchar(32) NULL, -- 创建人
                                                update_by varchar(32) NULL, -- 更新人
                                                "name" varchar(64) NULL, -- 巡检项目
                                                "content" varchar(64) NULL, -- 巡检内容
                                                sort int2 NOT NULL DEFAULT 1, -- 排序
                                                normal varchar(2) NULL DEFAULT 1, -- 是否正常 1-正常 0-不正常
                                                remark varchar(512) NULL, -- 备注
                                                task_id varchar(32) NULL, -- 巡检任务id
                                                CONSTRAINT tb_inspection_task_part_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_inspection_task_part IS '巡检任务关联信息';

-- Column comments

COMMENT ON COLUMN public.tb_inspection_task_part.id IS '主键id';
COMMENT ON COLUMN public.tb_inspection_task_part.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_inspection_task_part.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_inspection_task_part.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_inspection_task_part.create_by IS '创建人';
COMMENT ON COLUMN public.tb_inspection_task_part.update_by IS '更新人';
COMMENT ON COLUMN public.tb_inspection_task_part."name" IS '巡检项目';
COMMENT ON COLUMN public.tb_inspection_task_part."content" IS '巡检内容';
COMMENT ON COLUMN public.tb_inspection_task_part.sort IS '排序';
COMMENT ON COLUMN public.tb_inspection_task_part.normal IS '是否正常 1-正常 0-不正常';
COMMENT ON COLUMN public.tb_inspection_task_part.remark IS '备注';
COMMENT ON COLUMN public.tb_inspection_task_part.task_id IS '巡检任务id';


-- public.tb_inspection_task_project definition

-- Drop table

-- DROP TABLE public.tb_inspection_task_project;

CREATE TABLE public.tb_inspection_task_project (
                                                   id varchar(32) NOT NULL, -- 主键id
                                                   inspection_task_id varchar(32) NULL, -- 巡检任务Id
                                                   project_id varchar(32) NULL, -- 项目Id
                                                   CONSTRAINT tb_inspection_task_project_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_inspection_task_project IS '巡检项目关联id';

-- Column comments

COMMENT ON COLUMN public.tb_inspection_task_project.id IS '主键id';
COMMENT ON COLUMN public.tb_inspection_task_project.inspection_task_id IS '巡检任务Id';
COMMENT ON COLUMN public.tb_inspection_task_project.project_id IS '项目Id';


-- public.tb_inspection_task_vehicle definition

-- Drop table

-- DROP TABLE public.tb_inspection_task_vehicle;

CREATE TABLE public.tb_inspection_task_vehicle (
                                                   id varchar(32) NOT NULL, -- 主键id
                                                   vehicle_id varchar(32) NULL, -- 车辆Id
                                                   inspection_task_id varchar(32) NULL, -- 巡检任务Id
                                                   CONSTRAINT tb_inspection_task_vehicle_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_inspection_task_vehicle IS '巡检任务关联车辆';

-- Column comments

COMMENT ON COLUMN public.tb_inspection_task_vehicle.id IS '主键id';
COMMENT ON COLUMN public.tb_inspection_task_vehicle.vehicle_id IS '车辆Id';
COMMENT ON COLUMN public.tb_inspection_task_vehicle.inspection_task_id IS '巡检任务Id';

-- public.tb_scheduled_plan definition

-- Drop table

-- DROP TABLE public.tb_scheduled_plan;

CREATE TABLE public.tb_scheduled_plan (
                                          id varchar(32) NOT NULL, -- id
                                          sys_org_code varchar(32) NULL, -- 所属机构
                                          create_time timestamp NULL, -- 创建时间
                                          create_by varchar(32) NULL, -- 创建人
                                          update_time timestamp NULL, -- 更新时间
                                          update_by varchar(32) NULL, -- 更新人
                                          cron_expression varchar(32) NULL, -- cron表达式
                                          frequency_type varchar(2) NULL, -- 频率类型标识
                                          project_id text NULL, -- 关联项目
                                          plan_type varchar(2) NULL, -- 定时任务类型
                                          CONSTRAINT tb_scheduled_plan_pk PRIMARY KEY (id)
);
COMMENT ON TABLE public.tb_scheduled_plan IS '定时任务记录表';

-- Column comments

COMMENT ON COLUMN public.tb_scheduled_plan.id IS 'id';
COMMENT ON COLUMN public.tb_scheduled_plan.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_scheduled_plan.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_scheduled_plan.create_by IS '创建人';
COMMENT ON COLUMN public.tb_scheduled_plan.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_scheduled_plan.update_by IS '更新人';
COMMENT ON COLUMN public.tb_scheduled_plan.cron_expression IS 'cron表达式';
COMMENT ON COLUMN public.tb_scheduled_plan.frequency_type IS '频率类型标识';
COMMENT ON COLUMN public.tb_scheduled_plan.project_id IS '关联项目';
COMMENT ON COLUMN public.tb_scheduled_plan.plan_type IS '定时任务类型';



ALTER TABLE public.tb_contract ADD goods_name varchar(256) NULL;
COMMENT ON COLUMN public.tb_contract.goods_name IS '货物名称';

CREATE TABLE public.tb_contract_vehicle (
                                            id varchar(32) NULL,
                                            contract_id varchar(32) NULL,
                                            vehicle_brand varchar(32) NULL,
                                            vehicle_model varchar(32) NULL,
                                            vehicle_type varchar(32) NULL,
                                            quantity int4 NULL,
                                            total_price numeric(10, 2) NULL
);
COMMENT ON TABLE public.tb_contract_vehicle IS '合同车辆细项表';

-- Column comments

COMMENT ON COLUMN public.tb_contract_vehicle.id IS '主键id';
COMMENT ON COLUMN public.tb_contract_vehicle.contract_id IS '合同id';
COMMENT ON COLUMN public.tb_contract_vehicle.vehicle_brand IS '车辆品牌';
COMMENT ON COLUMN public.tb_contract_vehicle.vehicle_model IS '车辆型号';
COMMENT ON COLUMN public.tb_contract_vehicle.vehicle_type IS '车辆种类';
COMMENT ON COLUMN public.tb_contract_vehicle.quantity IS '数量';
COMMENT ON COLUMN public.tb_contract_vehicle.total_price IS '单价(万元含税)';


ALTER TABLE public.tb_insurance_company_archives ADD "type" varchar(16) NULL;
COMMENT ON COLUMN public.tb_insurance_company_archives."type" IS '供应商类型';
ALTER TABLE public.tb_insurance_company_archives ADD remark varchar(256) NULL;
COMMENT ON COLUMN public.tb_insurance_company_archives.remark IS '备注';

ALTER TABLE public.tb_repair_shop_archives ADD "type" varchar(16) NULL;
COMMENT ON COLUMN public.tb_repair_shop_archives."type" IS '供应商类型';
ALTER TABLE public.tb_repair_shop_archives ADD remark varchar(256) NULL;
COMMENT ON COLUMN public.tb_repair_shop_archives.remark IS '备注';

ALTER TABLE public.tb_vehicle_supplier_archives ADD "type" varchar(16) NULL;
COMMENT ON COLUMN public.tb_vehicle_supplier_archives."type" IS '供应商类型';
ALTER TABLE public.tb_vehicle_supplier_archives ADD remark varchar(256) NULL;


-- public.tb_vehicle_cost definition

-- Drop table

-- DROP TABLE public.tb_vehicle_cost;

CREATE TABLE public.tb_vehicle_cost (
                                        id varchar(32) NULL, -- id
                                        vehicle_type varchar(100) NULL, -- 车辆种类
                                        unit_price numeric(10, 4) NULL, -- 单价(万元)
                                        purchase_tax numeric(10, 4) NULL, -- 上牌费用/购置税(万元)
                                        insurance numeric(10, 4) NULL, -- 保险(万元/台/年)
                                        repair numeric(10, 4) NULL, -- 维修及易损件(万元)
                                        upkeep numeric(10, 4) NULL, -- 保养(万元)
                                        annual_audit numeric(10, 4) NULL, -- 年审(万元)
                                        one_year numeric(10, 4) NULL, -- 第一年
                                        two_year numeric(10, 4) NULL, -- 第二年
                                        three_year numeric(10, 4) NULL, -- 第三年
                                        four_year numeric(10, 4) NULL, -- 第四年
                                        five_year numeric(10, 4) NULL, -- 第五年
                                        six_year numeric(10, 4) NULL, -- 第六年
                                        seven_year numeric(10, 4) NULL, -- 第七年
                                        eight_year numeric(10, 4) NULL, -- 第八年
                                        sys_org_code varchar(32) NULL, -- 所属机构
                                        create_time timestamp NULL, -- 创建时间
                                        update_time timestamp NULL, -- 更新时间
                                        create_by varchar(32) NULL, -- 创建人
                                        update_by varchar(32) NULL -- 更新人
);
COMMENT ON TABLE public.tb_vehicle_cost IS '车辆成本表对';

-- Column comments

COMMENT ON COLUMN public.tb_vehicle_cost.id IS 'id';
COMMENT ON COLUMN public.tb_vehicle_cost.vehicle_type IS '车辆种类';
COMMENT ON COLUMN public.tb_vehicle_cost.unit_price IS '单价(万元)';
COMMENT ON COLUMN public.tb_vehicle_cost.purchase_tax IS '上牌费用/购置税(万元)';
COMMENT ON COLUMN public.tb_vehicle_cost.insurance IS '保险(万元/台/年)';
COMMENT ON COLUMN public.tb_vehicle_cost.repair IS '维修及易损件(万元)';
COMMENT ON COLUMN public.tb_vehicle_cost.upkeep IS '保养(万元)';
COMMENT ON COLUMN public.tb_vehicle_cost.annual_audit IS '年审(万元)';
COMMENT ON COLUMN public.tb_vehicle_cost.one_year IS '第一年';
COMMENT ON COLUMN public.tb_vehicle_cost.two_year IS '第二年';
COMMENT ON COLUMN public.tb_vehicle_cost.three_year IS '第三年';
COMMENT ON COLUMN public.tb_vehicle_cost.four_year IS '第四年';
COMMENT ON COLUMN public.tb_vehicle_cost.five_year IS '第五年';
COMMENT ON COLUMN public.tb_vehicle_cost.six_year IS '第六年';
COMMENT ON COLUMN public.tb_vehicle_cost.seven_year IS '第七年';
COMMENT ON COLUMN public.tb_vehicle_cost.eight_year IS '第八年';
COMMENT ON COLUMN public.tb_vehicle_cost.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_vehicle_cost.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_vehicle_cost.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_vehicle_cost.create_by IS '创建人';
COMMENT ON COLUMN public.tb_vehicle_cost.update_by IS '更新人';


-- public.tb_finance_base definition

-- Drop table

-- DROP TABLE public.tb_finance_base;

CREATE TABLE public.tb_finance_base (
                                        id varchar(32) NULL, -- id
                                        project_id varchar(32) NULL, -- 项目id
                                        year_limit int2 NULL, -- 租赁年限
                                        financing_ratio numeric(10, 4) NULL, -- 融资比例
                                        financing_rate numeric(10, 4) NULL, -- 融资利率（%）
                                        overhead_sharing numeric(10, 4) NULL, -- 管理费用分担（%）
                                        sys_org_code varchar(32) NULL, -- 所属机构
                                        create_time timestamp NULL, -- 创建时间
                                        update_time timestamp NULL, -- 更新时间
                                        create_by varchar(32) NULL, -- 创建人
                                        update_by varchar(32) NULL -- 更新人
);
COMMENT ON TABLE public.tb_finance_base IS '财务测算基本表';

-- Column comments

COMMENT ON COLUMN public.tb_finance_base.id IS 'id';
COMMENT ON COLUMN public.tb_finance_base.project_id IS '项目id';
COMMENT ON COLUMN public.tb_finance_base.year_limit IS '租赁年限';
COMMENT ON COLUMN public.tb_finance_base.financing_ratio IS '融资比例';
COMMENT ON COLUMN public.tb_finance_base.financing_rate IS '融资利率（%）';
COMMENT ON COLUMN public.tb_finance_base.overhead_sharing IS '管理费用分担（%）';
COMMENT ON COLUMN public.tb_finance_base.sys_org_code IS '所属机构';
COMMENT ON COLUMN public.tb_finance_base.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_finance_base.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_finance_base.create_by IS '创建人';
COMMENT ON COLUMN public.tb_finance_base.update_by IS '更新人';



-- public.tb_finance_measure_result definition

-- Drop table

-- DROP TABLE public.tb_finance_measure_result;

CREATE TABLE public.tb_finance_measure_result (
                                                  id varchar(32) NULL, -- id
                                                  base_id varchar(32) NULL, -- 基础表id
                                                  car_cost_recovery numeric(10, 4) NULL, -- 购车成本回收期(年)
                                                  financing_cost numeric(10, 4) NULL, -- 融资成本(万元)
                                                  internal_rate numeric(10, 4) NULL, -- 内部收益率(%)
                                                  vat_and_additional numeric(10, 4) NULL, -- 实缴增值税及附加(万元)
                                                  income_tax numeric(10, 4) NULL, -- 所得税(万元)
                                                  net_profit numeric(10, 4) NULL, -- 净利润(万元)
                                                  salvage_value numeric(10, 4) NULL, -- 残值(万元)
                                                  rental_income_json text NULL, -- 租金收入(万元)
                                                  manage_cost_json text NULL, -- 管理、运维、保险费用(万元)
                                                  income_and_tax_json text NULL, -- 收入与税金表(万元)
                                                  cost_and_profit_json text NULL -- 成本与利润表(万元)
);
COMMENT ON TABLE public.tb_finance_measure_result IS '财务测算测算结果关联表';

-- Column comments

COMMENT ON COLUMN public.tb_finance_measure_result.id IS 'id';
COMMENT ON COLUMN public.tb_finance_measure_result.base_id IS '基础表id';
COMMENT ON COLUMN public.tb_finance_measure_result.car_cost_recovery IS '购车成本回收期(年)';
COMMENT ON COLUMN public.tb_finance_measure_result.financing_cost IS '融资成本(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.internal_rate IS '内部收益率(%)';
COMMENT ON COLUMN public.tb_finance_measure_result.vat_and_additional IS '实缴增值税及附加(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.income_tax IS '所得税(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.net_profit IS '净利润(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.salvage_value IS '残值(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.rental_income_json IS '租金收入(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.manage_cost_json IS '管理、运维、保险费用(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.income_and_tax_json IS '收入与税金表(万元)';
COMMENT ON COLUMN public.tb_finance_measure_result.cost_and_profit_json IS '成本与利润表(万元)';



-- public.tb_finance_vehicle_cost definition

-- Drop table

-- DROP TABLE public.tb_finance_vehicle_cost;

CREATE TABLE public.tb_finance_vehicle_cost (
                                                id varchar(32) NULL, -- id
                                                base_id varchar(32) NULL, -- 基础表id
                                                vehicle_type varchar(100) NULL, -- 车辆种类
                                                unit_price numeric(10, 4) NULL, -- 单价(万元)
                                                purchase_tax numeric(10, 4) NULL, -- 上牌费用/购置税(万元)
                                                insurance numeric(10, 4) NULL, -- 保险(万元/台/年)
                                                repair numeric(10, 4) NULL, -- 维修及易损件(万元)
                                                upkeep numeric(10, 4) NULL, -- 保养(万元)
                                                annual_audit numeric(10, 4) NULL, -- 年审(万元)
                                                quantity int4 NULL, -- 数量
                                                salvage_value numeric(10, 4) NULL, -- 残值(万元/台)
                                                one_year numeric(10, 4) NULL, -- 第一年
                                                two_year numeric(10, 4) NULL, -- 第二年
                                                three_year numeric(10, 4) NULL, -- 第三年
                                                four_year numeric(10, 4) NULL, -- 第四年
                                                five_year numeric(10, 4) NULL, -- 第五年
                                                six_year numeric(10, 4) NULL, -- 第六年
                                                seven_year numeric(10, 4) NULL, -- 第七年
                                                eight_year numeric(10, 4) NULL -- 第八年
);
COMMENT ON TABLE public.tb_finance_vehicle_cost IS '财务测算车辆费用关联表';

-- Column comments

COMMENT ON COLUMN public.tb_finance_vehicle_cost.id IS 'id';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.base_id IS '基础表id';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.vehicle_type IS '车辆种类';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.unit_price IS '单价(万元)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.purchase_tax IS '上牌费用/购置税(万元)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.insurance IS '保险(万元/台/年)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.repair IS '维修及易损件(万元)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.upkeep IS '保养(万元)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.annual_audit IS '年审(万元)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.quantity IS '数量';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.salvage_value IS '残值(万元/台)';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.one_year IS '第一年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.two_year IS '第二年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.three_year IS '第三年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.four_year IS '第四年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.five_year IS '第五年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.six_year IS '第六年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.seven_year IS '第七年';
COMMENT ON COLUMN public.tb_finance_vehicle_cost.eight_year IS '第八年';


CREATE TABLE public.tb_project_user_rel (
	id varchar(32) NOT NULL,
	project_id varchar(32) NULL, -- 项目id
	user_id varchar(32) NULL, -- 用户id
	create_time timestamp(6) NULL, -- 创建时间
	create_by varchar(32) NULL, -- 创建人
	update_time timestamp(6) NULL, -- 更新时间
	update_by varchar(32) NULL, -- 更新人
	CONSTRAINT tb_project_user_pkey PRIMARY KEY (id)
);

-- Column comments

COMMENT ON COLUMN public.tb_project_user_rel.project_id IS '项目id';
COMMENT ON COLUMN public.tb_project_user_rel.user_id IS '用户id';
COMMENT ON COLUMN public.tb_project_user_rel.create_time IS '创建时间';
COMMENT ON COLUMN public.tb_project_user_rel.create_by IS '创建人';
COMMENT ON COLUMN public.tb_project_user_rel.update_time IS '更新时间';
COMMENT ON COLUMN public.tb_project_user_rel.update_by IS '更新人';


INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714833496767037441', '1702865172494635010', '保养', '', NULL, NULL, NULL, 2, 'UpkeepBill:repairing', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:39:00.351', 'adminLeader', '2023-10-19 10:39:00.351', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714833190222135298', '1702865172494635010', '审核', '', NULL, NULL, NULL, 2, 'UpkeepBill:checkOffer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:37:47.265', 'adminLeader', '2023-10-19 10:37:47.265', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714833017202900993', '1702865172494635010', '报价', '', NULL, NULL, NULL, 2, 'UpkeepBill:offer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:37:06.015', 'adminLeader', '2023-10-19 10:37:06.015', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714832922499710977', '1702865172494635010', '派单', '', NULL, NULL, NULL, 2, 'UpkeepBill:send', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:36:43.435', 'adminLeader', '2023-10-19 10:36:43.435', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714827702247981057', '1702611066496593922', '作废', '', NULL, NULL, NULL, 2, 'UpkeepPlan:cancel', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:15:58.830', 'adminLeader', '2023-10-19 10:15:58.830', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714827627677450242', '1702611066496593922', '编辑', '', NULL, NULL, NULL, 2, 'UpkeepPlan:edit', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:15:41.051', 'adminLeader', '2023-10-19 10:15:41.051', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714827144829173761', '1702611066496593922', '审批', '', NULL, NULL, NULL, 2, 'UpkeepPlan:leaderCheckOffer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:13:45.930', 'yubowen', '2023-10-19 15:48:23.563', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714826902809444353', '1702611066496593922', '查看', '', NULL, NULL, NULL, 2, 'UpkeepPlan:view', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 10:12:48.227', 'adminLeader', '2023-10-19 10:14:16.975', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714814366764589057', '1701116944293138433', '作废', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:cancel', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:22:59.403', 'adminLeader', '2023-10-19 09:33:53.865', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714813971195584514', '1701116944293138433', '查看', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:view', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:21:25.091', 'adminLeader', '2023-10-19 09:34:11.531', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714813702047096834', '1701116944293138433', '确认', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:confirmResult', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:20:20.922', 'adminLeader', '2023-10-19 09:33:38.318', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714813480818532354', '1701116944293138433', '维修', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:repairing', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:19:28.174', 'adminLeader', '2023-10-19 09:30:13.278', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714813300861919233', '1701116944293138433', '审批', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:leaderCheckOffer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:18:45.272', 'adminLeader', '2023-10-19 09:54:38.216', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714813137841905665', '1701116944293138433', '审核', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:checkOffer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:18:06.406', 'adminLeader', '2023-10-19 09:30:01.648', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714812828595871746', '1701116944293138433', '报价', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:offer', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:16:52.675', 'adminLeader', '2023-10-19 09:54:23.853', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714812708307427330', '1701116944293138433', '派单', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:send', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:16:23.997', 'adminLeader', '2023-10-19 09:29:33.327', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1714812533069406209', '1701116944293138433', '提交', '', NULL, NULL, NULL, 2, 'VehiclesMaintain:submit', '1', 1.0, NULL, 1, NULL, '1', NULL, NULL, 'adminLeader', '2023-10-19 09:15:42.215', 'adminLeader', '2023-10-19 09:29:51.689', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1713743963266781186', '1712281696415252481', '车辆巡检', '/customerService/inspection', 'customerService/inspection/index', NULL, NULL, 1, NULL, NULL, 0.0, 'DocumentChecked', 0, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-16 10:29:35.320', 'chenyufeng', '2023-10-16 10:29:35.320', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1713740570959912961', '1712281696415252481', '客户回访', '/customerService/follow', 'customerService/follow/index', NULL, NULL, 1, NULL, NULL, 0.0, 'Handbag', 1, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-16 10:16:06.531', 'chenyufeng', '2023-10-16 10:16:06.531', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1712311358352785409', '1712281696415252481', '意见反馈', '/customerService/feedback', 'customerService/feedback/index', NULL, NULL, 1, NULL, NULL, 3.0, 'Warning', 0, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-12 11:36:55.694', 'chenyufeng', '2023-10-12 11:37:07.981', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1712282149702074369', '1712281696415252481', '客户培训', '/customerService/training', 'customerService/training/index', NULL, NULL, 1, NULL, NULL, 0.0, 'Finished', 1, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-12 09:40:51.809', 'chenyufeng', '2023-10-16 09:41:13.024', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1712281696415252481', '', '客户服务', '/customerService', NULL, NULL, NULL, 0, NULL, NULL, 4.0, 'Dessert', 0, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-12 09:39:03.738', 'weibao_leader', '2023-10-18 15:19:28.807', 0, 0);
INSERT INTO public.sys_permission (id, parent_id, "name", url, component, component_name, redirect, menu_type, perms, perms_type, sort_no, icon, is_leaf, description, status, del_flag, rule_flag, create_by, create_time, update_by, update_time, internal_or_external, hidden) VALUES('1711946741130723329', '1699702223404421121', '客户回访模板', '/baseData/Template/CustomerFollowTemplate', 'baseData/Template/CustomerFollowTemplate/index', NULL, NULL, 1, NULL, NULL, 0.0, 'Tickets', 1, NULL, NULL, NULL, NULL, 'chenyufeng', '2023-10-11 11:28:04.173', 'chenyufeng', '2023-10-16 09:27:24.623', 0, 0);


INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714535024125841409', '频率类型', 'frequency_type', '每周1；
每月2；
每月两次3；
每半年4；
每年5；', 0, 'chenyufeng', '2023-10-18 14:52:58.928', 'chenyufeng', '2023-10-20 17:23:45.521', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714197385170423810', '结果状态2', 'opinion_status', '1未完成，2已完成', 0, 'admin', '2023-10-17 16:31:19.528', 'admin', '2023-10-18 16:01:47.998', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714153961331396609', '供应商类型', 'supplier_type', '', 0, 'adminLeader', '2023-10-17 13:38:46.478', 'adminLeader', '2023-10-17 13:38:46.478', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714123765605765122', '巡检类型', 'inspection_type', '', 0, 'chenyufeng', '2023-10-17 11:38:47.256', 'chenyufeng', '2023-10-17 11:38:47.256', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714121501826650113', '客户满意度', 'pleased_status', '1满意，2不满意', 0, 'admin', '2023-10-17 11:29:47.529', 'admin', '2023-10-17 11:29:47.529', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1714121063924535298', '意见分类', 'opinion_type', '1客户服务，2服务需求，3其他', 0, 'admin', '2023-10-17 11:28:03.125', 'admin', '2023-10-17 11:28:03.125', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1713892858458312705', '巡检结果', 'inspection_result', '', 0, 'chenyufeng', '2023-10-16 20:21:14.702', 'chenyufeng', '2023-10-16 20:21:14.702', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1713106236410273793', '回访方式', 'review_type', '', 0, 'admin', '2023-10-14 16:15:29.391', 'admin', '2023-10-14 16:15:29.391', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1713105899179843585', '结果状态', 'result_status', '0未开始，1未完成，2已完成', 0, 'admin', '2023-10-14 16:14:08.988', 'admin', '2023-10-17 11:31:09.387', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1712430019545661441', '事件分类', 'event_category', '', 0, 'weibao_1', '2023-10-12 19:28:26.727', 'weibao_1', '2023-10-12 19:28:26.727', 0);
INSERT INTO public.sys_dict (id, dict_name, dict_code, description, del_flag, create_by, create_time, update_by, update_time, "type") VALUES('1710856749092507650', '项目状态', 'project_status', '', 0, 'weibao_leader', '2023-10-08 11:16:49.821', NULL, '2023-10-08 11:16:49.821', 0);


INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1716369065439461377', '1698869263734734849', 'BB', '16', NULL, NULL, 1, 'weibao_leader', '2023-10-23 16:20:48.452', 'weibao_leader', '2023-10-23 16:20:48.452');

INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714573090558410754', '1697444619626110978', '其它', '99', '', 99, 1, 'weibao_1', '2023-10-18 17:24:14.679', 'weibao_1', '2023-10-18 17:24:14.679');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714535530558689282', '1714535024125841409', '每年', '5', '', 5, 1, 'chenyufeng', '2023-10-18 14:54:59.671', 'chenyufeng', '2023-10-18 14:54:59.671');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714535487881646081', '1714535024125841409', '每半年', '4', '', 4, 1, 'chenyufeng', '2023-10-18 14:54:49.496', 'chenyufeng', '2023-10-18 14:54:49.496');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714535284147523586', '1714535024125841409', '每月两次', '3', '', 3, 1, 'chenyufeng', '2023-10-18 14:54:00.922', 'chenyufeng', '2023-10-18 14:54:00.922');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714535225548902401', '1714535024125841409', '每月', '2', '', 2, 1, 'chenyufeng', '2023-10-18 14:53:46.951', 'chenyufeng', '2023-10-18 14:53:46.951');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714535143369904130', '1714535024125841409', '每周', '1', '', 1, 1, 'chenyufeng', '2023-10-18 14:53:27.359', 'chenyufeng', '2023-10-18 14:53:27.359');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714534846773891073', '1714534762803924993', '巡检任务', '2', '', 2, 1, 'chenyufeng', '2023-10-18 14:52:16.645', 'chenyufeng', '2023-10-18 14:52:16.645');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714534804386254850', '1714534762803924993', '客户回访', '1', '', 1, 1, 'chenyufeng', '2023-10-18 14:52:06.538', 'chenyufeng', '2023-10-18 14:52:06.538');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714197479441600513', '1714197385170423810', '已完成', '2', '', 2, 1, 'admin', '2023-10-17 16:31:42.004', 'admin', '2023-10-17 16:31:42.004');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714197436600979458', '1714197385170423810', '未完成', '1', '', 1, 1, 'admin', '2023-10-17 16:31:31.790', 'admin', '2023-10-17 16:31:31.790');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714193559134703618', '1714153961331396609', '车辆供应商', '3', '', 3, 1, 'adminLeader', '2023-10-17 16:16:07.330', 'adminLeader', '2023-10-17 16:16:07.330');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714154125404180482', '1714153961331396609', '其它', '9', '', 9, 1, 'adminLeader', '2023-10-17 13:39:25.596', 'adminLeader', '2023-10-17 13:39:25.596');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714154064070873089', '1714153961331396609', '保险公司', '2', '', 2, 1, 'adminLeader', '2023-10-17 13:39:10.973', 'adminLeader', '2023-10-17 13:39:10.973');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714154010924847106', '1714153961331396609', '维修厂', '1', '', 1, 1, 'adminLeader', '2023-10-17 13:38:58.302', 'adminLeader', '2023-10-17 13:38:58.302');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714130273303302145', '1697501565750706177', '其它', '9', '', 9, 1, 'adminLeader', '2023-10-17 12:04:38.812', 'adminLeader', '2023-10-17 12:04:38.812');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714123867535740929', '1714123765605765122', '回访巡检', '2', '', 1, 1, 'chenyufeng', '2023-10-17 11:39:11.558', 'chenyufeng', '2023-10-17 11:39:11.558');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714123827140399105', '1714123765605765122', '线上巡检', '1', '', 1, 1, 'chenyufeng', '2023-10-17 11:39:01.927', 'chenyufeng', '2023-10-17 11:39:01.927');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714121592859824129', '1714121501826650113', '不满意', '2', '', 2, 1, 'admin', '2023-10-17 11:30:09.234', 'admin', '2023-10-17 11:30:09.234');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714121554154786818', '1714121501826650113', '满意', '1', '', 1, 1, 'admin', '2023-10-17 11:30:00.003', 'admin', '2023-10-17 11:30:00.003');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714121230249660418', '1714121063924535298', '其他', '3', '', 3, 1, 'admin', '2023-10-17 11:28:42.781', 'admin', '2023-10-17 11:28:42.781');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714121188667330561', '1714121063924535298', '服务需求', '2', '', 2, 1, 'admin', '2023-10-17 11:28:32.867', 'admin', '2023-10-17 11:28:32.867');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1714121139182931970', '1714121063924535298', '客户服务', '1', '', 1, 1, 'admin', '2023-10-17 11:28:21.069', 'admin', '2023-10-17 11:28:21.069');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713892959318740994', '1713892858458312705', '不合格', '0', '', 1, 1, 'chenyufeng', '2023-10-16 20:21:38.749', 'chenyufeng', '2023-10-16 20:21:38.749');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713892910576734209', '1713892858458312705', '合格', '1', '', 1, 1, 'chenyufeng', '2023-10-16 20:21:27.127', 'chenyufeng', '2023-10-16 20:21:27.127');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713793205385650177', '1713793008228196353', '已完成', '2', '', 2, 1, 'chenyufeng', '2023-10-16 13:45:15.558', 'chenyufeng', '2023-10-16 13:45:15.558');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713793166034690049', '1713793008228196353', '未完成', '1', '', 1, 1, 'chenyufeng', '2023-10-16 13:45:06.176', 'chenyufeng', '2023-10-16 13:45:06.176');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713793128587943938', '1713793008228196353', '未开始', '0', '', 1, 1, 'chenyufeng', '2023-10-16 13:44:57.248', 'chenyufeng', '2023-10-16 13:44:57.248');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713106325744754690', '1713106236410273793', '电话回访', '2', '', 2, 1, 'admin', '2023-10-14 16:15:50.690', 'admin', '2023-10-14 16:15:50.690');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713106285060005889', '1713106236410273793', '上门回访', '1', '', 1, 1, 'admin', '2023-10-14 16:15:40.990', 'admin', '2023-10-14 16:15:40.990');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713106135755366402', '1713105899179843585', '已完成', '2', '', 3, 1, 'admin', '2023-10-14 16:15:05.393', 'admin', '2023-10-14 16:15:05.393');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713106081929863170', '1713105899179843585', '未完成', '1', '', 2, 1, 'admin', '2023-10-14 16:14:52.561', 'admin', '2023-10-14 16:14:52.561');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1713106000287735809', '1713105899179843585', '未开始', '0', '', 1, 1, 'admin', '2023-10-14 16:14:33.094', 'admin', '2023-10-14 16:14:33.094');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1712430131571326978', '1712430019545661441', '其它', '10', '', 1, 1, 'weibao_1', '2023-10-12 19:28:53.436', 'weibao_1', '2023-10-12 19:28:53.436');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1712430083458465794', '1712430019545661441', '特情类', '2', '', 1, 1, 'weibao_1', '2023-10-12 19:28:41.965', 'weibao_1', '2023-10-12 19:28:41.965');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1712430058259087361', '1712430019545661441', '培训类', '1', '', 1, 1, 'weibao_1', '2023-10-12 19:28:35.958', 'weibao_1', '2023-10-12 19:28:35.958');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711359157749018625', '1698869263734734849', '你好你好', '15', NULL, NULL, 1, 'yubowen', '2023-10-09 20:33:13.380', NULL, '2023-10-09 20:33:13.380');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711356925850808322', '1698869263734734849', '测试行为', '14', NULL, NULL, 1, 'yubowen', '2023-10-09 20:24:21.254', NULL, '2023-10-09 20:24:21.254');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711356133899104258', '1698869263734734849', 'abc', '13', NULL, NULL, 1, 'yubowen', '2023-10-09 20:21:12.438', NULL, '2023-10-09 20:21:12.438');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711352434854326273', '1698869263734734849', '超速889', '12', NULL, NULL, 1, 'yubowen', '2023-10-09 20:06:30.516', NULL, '2023-10-09 20:06:30.516');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711351836578803713', '1698869263734734849', '超速88', '11', NULL, NULL, 1, 'yubowen', '2023-10-09 20:04:07.876', NULL, '2023-10-09 20:04:07.876');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711289316304297986', '1698869263734734849', 'hhjj', '10', NULL, NULL, 1, NULL, '2023-10-09 15:55:41.882', NULL, '2023-10-09 15:55:41.882');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1711285371527831554', '1698869263734734849', 'fghh', '9', NULL, NULL, 1, NULL, '2023-10-09 15:40:01.372', NULL, '2023-10-09 15:40:01.372');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1710927879753027586', '1698869263734734849', 'AQ', '8', NULL, NULL, 1, 'weibao_leader', '2023-10-08 15:59:28.692', NULL, '2023-10-08 15:59:28.692');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1710926757101424641', '1698869263734734849', 'AA', '7', NULL, NULL, 1, 'weibao_leader', '2023-10-08 15:55:01.031', NULL, '2023-10-08 15:55:01.031');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1710856827790233601', '1710856749092507650', '结束', '0', '', 1, 1, 'weibao_leader', '2023-10-08 11:17:08.584', NULL, '2023-10-08 11:17:08.584');
INSERT INTO public.sys_dict_item (id, dict_id, item_text, item_value, description, sort_order, status, create_by, create_time, update_by, update_time) VALUES('1710856797847097346', '1710856749092507650', '进行中', '1', '', 1, 1, 'weibao_leader', '2023-10-08 11:17:01.445', NULL, '2023-10-08 11:17:01.445');

